{% extends "base.html" %}
{% block page %}
<div class="page">
  <div>
    Logged in as <strong>{{user}}</strong>
  </div>

  {% for client in clients %}
  <pre>
    {{ client | tojson(2) }}
  </pre>

  <hr>
  {% endfor %}

  <br>

  <a href="/oauth/create-client">Create Client</a>
  | <a href="/oauth/logout">Log out</a>
</div>
{% endblock %}