{% extends "base.html" %}
{% block page %}
<div class="page">
  <form action="" method="post">
    <label>
      <span>Client Name</span>
      <input type="text" name="client_name" value="test">
    </label>
    <label>
      <span>Client ID</span>
      <input type="text" name="client_id" value="test">
    </label>
    <label>
      <span>Client Secret</span>
      <input type="text" name="client_secret" value="test">
    </label>
    <label>
      <span>Client URI</span>
      <input type="url" name="client_uri" value="http://localhost:5001">
    </label>
    <label>
      <span>Allowed Scope</span>
      <input type="text" name="scope" value="openid profile">
    </label>
    <label>
      <span>Redirect URIs</span>
      <textarea name="redirect_uri" cols="30" rows="3">http://localhost:5001</textarea>
    </label>
    <label>
      <span>Allowed Origins</span>
      <textarea name="allowed_origins" cols="30" rows="3">http://localhost:5001</textarea>
    </label>
    <label>
      <span>Allowed Grant Types</span>
      <textarea name="grant_type" cols="30" rows="3">authorization_code</textarea>
    </label>
    <label>
      <span>Allowed Response Types</span>
      <textarea name="response_type" cols="30" rows="3">code</textarea>
    </label>
    <label>
      <span>Permission Groups</span>
      <textarea name="permission_groups" cols="30" rows="3">Group1
Group2
Group3</textarea>
    </label>
    <button>Submit</button>
  </form>

  <hr>

  <a href="/">Home</a>
</div>
{% endblock %}
