html, body {
  margin: 0px;
  padding: 0px;
  background-color: #eee;
  font-family: sans-serif;
}

/* dialog is centered horizontally and vertically */
.container {
  display: flex;
  justify-content: center;
  align-items: center;
  height: calc(100vh - 70px);

  padding: 0px;
  margin: 0px;
}

.container .dialog {
  width: 400px;
  border:1px solid black;
  border-radius: 15px;
  filter: drop-shadow(10px 10px 4px #aaa);
}

.dialog .title {
  padding: 5px;
  background-color: #004d99;
  color:white;
  font-weight: bold;
  border-top-left-radius: 15px;
  border-top-right-radius: 15px;
}

/* content is centered horizontally and vertically */
.dialog .content {
  display: flex;
  justify-content: center;
  align-items: center;
  min-height: 200px;
  padding: 0px;
  background-color: white;
  border-bottom-left-radius: 15px;
  border-bottom-right-radius: 15px;
}

.dialog .content form {
  display: block;
  text-align: center;
}

.banner {
  display: flex;
  justify-content: center;
  align-items: center;
  height: 50px;

  padding: 10px;
  font-weight: bold;
  font-size: 18pt;
  background: repeating-linear-gradient(
    -45deg,
    #f6ba52,
    #f6ba52 30px,
    #ffd180 30px,
    #ffd180 60px
  );
}

.page {
  padding: 15px;
}

pre {
  white-space:wrap
}

label, label > span { display: block; }
label { margin: 15px 0; }
