{% extends "base.html" %}
{% block page %}
<div class="container">
  <div class="dialog">

    <div class="title">{% block title %}{% endblock %}</div>
    <div class="content">{% block content %}{% endblock %}</div>

  </div>
</div>
{% endblock %}
