{% extends "dialog.html" %}
{% block title %}Login{% endblock %}
{% block content %}
<form action="?{{ args }}" method="post">
  <input type="text" name="username" placeholder="username"><br>
  <button type="submit">Login</button>
</form>
{% endblock %}
